<?php

namespace Drupal\regex_field_validator\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldWidget\StringTextfieldWidget;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'regex_text_widget' widget.
 *
 * @FieldWidget(
 *   id = "regex_text_widget",
 *   label = @Translation("Text field with regex validation"),
 *   field_types = {
 *     "string",
 *     "string_long",
 *     "text",
 *     "text_long"
 *   }
 * )
 */
class RegexTextWidget extends StringTextfieldWidget {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'regex_pattern' => '',
      'regex_message' => 'The input does not match the required pattern.',
      'regex_enabled' => FALSE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);

    $elements['regex_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable regex validation'),
      '#default_value' => $this->getSetting('regex_enabled'),
      '#description' => $this->t('Check to enable regular expression validation for this field.'),
    ];

    $elements['regex_pattern'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Regular Expression Pattern'),
      '#default_value' => $this->getSetting('regex_pattern'),
      '#description' => $this->t('Enter a valid regular expression pattern (without delimiters). Example: ^[A-Za-z0-9]+$ for alphanumeric only.'),
      '#states' => [
        'visible' => [
          ':input[name$="[settings_edit_form][settings][regex_enabled]"]' => ['checked' => TRUE],
        ],
        'required' => [
          ':input[name$="[settings_edit_form][settings][regex_enabled]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $elements['regex_message'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Validation Error Message'),
      '#default_value' => $this->getSetting('regex_message'),
      '#description' => $this->t('Custom error message to display when validation fails.'),
      '#states' => [
        'visible' => [
          ':input[name$="[settings_edit_form][settings][regex_enabled]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();

    if ($this->getSetting('regex_enabled')) {
      $pattern = $this->getSetting('regex_pattern');
      $summary[] = $this->t('Regex validation: @pattern', ['@pattern' => $pattern ?: 'Not set']);
    } else {
      $summary[] = $this->t('Regex validation: Disabled');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);

    if ($this->getSetting('regex_enabled') && !empty($this->getSetting('regex_pattern'))) {
      $element['value']['#element_validate'][] = [$this, 'validateRegex'];
      $element['value']['#regex_pattern'] = $this->getSetting('regex_pattern');
      $element['value']['#regex_message'] = $this->getSetting('regex_message');
    }

    return $element;
  }

  /**
   * Validates the regex pattern against the field value.
   */
  public function validateRegex($element, FormStateInterface $form_state, $form) {
    $value = $element['#value'];
    $pattern = $element['#regex_pattern'];
    $message = $element['#regex_message'];

    if (!empty($value) && !empty($pattern)) {
      // Add delimiters to the pattern if not present
      if (substr($pattern, 0, 1) !== '/' && substr($pattern, 0, 1) !== '#') {
        $pattern = '/' . $pattern . '/';
      }

      if (!preg_match($pattern, $value)) {
        $form_state->setError($element, $message);
      }
    }
  }

}